/*
################################################################################################################################################################
	This is a patch needed to convert a "window" in X to a "Display".
	It is needed to make xpseudoroot, a program that came with X11R3
	run properly in later releases of X.  Specifically versions 4 and 5.
	Apparently in X11R3, they included some extra code in
	XOpenDis.c to allow you to connect to a window of a display instead of
	of just to the Display.Screen_number.  Since xpseudoroot relied upon
	that portion of the XOpenDis routine, in later versions, it would not
	work.
	
	I am hoping that the changes that I make will be global enough to work
	on all X systems.  If it is not, my apologies.  Good Luck, and I hope it
	works for you!

	The routine is free and may be distributed in whole or part by anyone,
	provided that it always retains all the comments from the two lines of
	'#' above to the two lines of '#' below.  It was written exclusively for
	the use of employees at Wright State University in Dayton, Ohio.
	Any questions about it can be sent to njenkins@discover.wright.edu  or
	njenkins@desire.wright.edu   or desire::njenkins  if on Decnet,   or
	njenkins@wsu.bitnet

	happy hunting...

	P.S. - if you have a utility like the textedit tool of SunOS,
	you might want to use it, so that you can view this as more
	than just 80 columns, if you wish to mod it.

	P.P.S. Please feel free to make mods, and send them to me.
################################################################################################################################################################
*/
#include <X11/Xlib.h>
#include <X11/Xauth.h>
#include <sys/wait.h>
#include <stdio.h>
#include <X11/Xos.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/bitmaps/gray1>
#include <ctype.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <X11/Xlibint.h>
#include <sys/errno.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <stropts.h>
#include <aio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <poll.h>
#include <fcntl.h>
#include <values.h>
#include <netinet/in.h>
#define X_UNIX_DIR	"/tmp/.X11-unix"
#define X_UNIX_PATH	"/tmp/.X11-unix/X"
#include "PseudoRoot.h"
#if defined(EAGAIN) && defined(EWOULDBLOCK)
#define ETEST(err) (err == EAGAIN || err == EWOULDBLOCK)
#else
#ifdef EAGAIN
#define ETEST(err) (err == EAGAIN)
#else 
#define ETEST(err) (err == EWOULDBLOCK)
#endif
#endif

typedef struct {	/*This is for maintaining a pair of sockets*/
	struct aiocb *aio[2];
	long int aiotable[2];
	long int offset[2];
	long int written[2];
	long int fd[2];
	} fdstruct;


static long int unixDomainConnection = -1; /*Stuff I copied out of other X stuff*/
static struct sockaddr_un unsock;
static long int xauth_namelen = 0;
static char *xauth_name = NULL;	 /* NULL means use default mechanism */
static long int xauth_datalen = 0;
static char *xauth_data = NULL;	 /* NULL means get default data */

static char *default_xauth_names[] = {
#ifdef SECURE_RPC
    "SUN-DES-1",
#endif
#ifdef HASXDMAUTH
    "XDM-AUTHORIZATION-1",
#endif
    "MIT-MAGIC-COOKIE-1"
};

static long int default_xauth_lengths[] = {
#ifdef SECURE_RPC
    9,	    /* strlen ("SUN-DES-1") */
#endif
#ifdef HASXDMAUTH
    19,	    /* strlen ("XDM-AUTHORIZATION-1") */
#endif
    18	    /* strlen ("MIT-MAGIC-COOKIE-1") */
};

#define NUM_DEFAULT_AUTH    (sizeof (default_xauth_names) / sizeof (default_xauth_names[0]))
    
static char **xauth_names = default_xauth_names;
static long int  *xauth_lengths = default_xauth_lengths;

static long int  xauth_names_length = NUM_DEFAULT_AUTH;

			/*My variables */

long int nick_exec_number=0; /*for the -f(ork) command-line flag*/
long int unix_sock=0;        /*for the -u(nix socket) command-line flag*/
long int fdset;		/*global for which fd set I am on when SIGPIPE occur*/
long int fdsets;		/*How many sets are there (0 counts as a set*/
long int first_open;		/*First open connection socket*/
long int allowed_open;	/*How Many Total descriptors allowed?*/
long int exec_flag;		/*variable for storing pid of fork() for -f flag*/
long int daemon;		/*variable for storing pid of fork() - socket daemon*/
char *name;		/*I forget*/
fdstruct *fds;		/*array of fd sets*/
long int signalmask;

static OutOfMemory();   /*No Clue - I copied this */
void readme(),writeme(); /*Just little functions defined at the end */


long int pad(i)		/*Useful for determining how many bytes I need to pad*/
long int i;			/*a message with (X communication is in 4 byte incr.*/
{
switch(i%4)
	{
	case 3: return 1; break;
	case 2: return 2; break;
	case 1: return 3; break;
	default: return 0; break;
	}
}

long int change_endian(in,size)
long int in,size;
{
long int a;
if (size == 2) {
	a=(in>>8)&255;
	a+=((in&255)*256);
	return a;
}
else {
	a=((in>>24)&255);
	a+=(((in>>16)&255)*256);
	a+=(((in>>8)&255)*256*256);
	a+=((in&255)*256*256*256);
	return a;
}
}


char *itoa(i)		/*changes int to ascii*/
long int *i;
{
long int j,k;
char *display;

for (j=*i,k=1;j/10>=1;j/=10,k++);
k++;
display= (char *)malloc(k);
display[--k]='\0';
for (j=*i,k--;j/10>=1;j/=10,k--)
	{
	switch (j%10)
		{
		case 0: display[k]='0';break;
		case 1: display[k]='1';break;
		case 2: display[k]='2';break;
		case 3: display[k]='3';break;
		case 4: display[k]='4';break;
		case 5: display[k]='5';break;
		case 6: display[k]='6';break;
		case 7: display[k]='7';break;
		case 8: display[k]='8';break;
		case 9: display[k]='9';break;
		}
	}
	switch (j%10)
		{
		case 0: display[k]='0';break;
		case 1: display[k]='1';break;
		case 2: display[k]='2';break;
		case 3: display[k]='3';break;
		case 4: display[k]='4';break;
		case 5: display[k]='5';break;
		case 6: display[k]='6';break;
		case 7: display[k]='7';break;
		case 8: display[k]='8';break;
		case 9: display[k]='9';break;
		}
return(display);
}

long int open_tcp_socket(display)	/*display is actually a number*/
char *display;			/*addressing is server:display.0 */
{
    struct sockaddr_in insock;
    long int request;
    long int retry;

    if ((request=socket(AF_INET,SOCK_STREAM,0))<0) return -1;
    bzero ((char *)&insock, sizeof (insock));
    insock.sin_family = AF_INET;
    insock.sin_port = htons ((unsigned long)(X_TCP_PORT + atoi (display)));
    insock.sin_addr.s_addr = htonl(INADDR_ANY);
    retry = 20;
    while (bind(request, (struct sockaddr *) &insock, sizeof (insock)))
    {
	if (--retry == 0) {
	    close (request);
	    return -1;
	}
    }
    if (listen (request, 5)) {
	close (request);
	return -1;
    }
    return request;
}


long int open_unix_socket (display) 	/*display is actually a number*/
char *display;			/*addressing is :display.0 */
{
    long int oldUmask;
    long int i,fd;

    fd=-1;
    bzero ((char *) &unsock, sizeof (unsock));
    unsock.sun_family = AF_UNIX;
    oldUmask = umask (0);
#ifdef X_UNIX_DIR
    mkdir (X_UNIX_DIR, 0777);
    chmod (X_UNIX_DIR, 0777);
#endif
    strcpy (unsock.sun_path, X_UNIX_PATH);
    strcat (unsock.sun_path, display);

    if ((fd = socket (AF_UNIX, SOCK_STREAM, 0)) < 0)
    {
    return -1;
    }

    if (bind(fd, (struct sockaddr *)&unsock, strlen(unsock.sun_path)+2))
    {
        return -1;
    }
    if (listen (fd, 5))
    {
        return -1;
    }
    (void)umask(oldUmask);
    return (fd);
}

void XChuckWinScreen(nameofit)	/*removes file /tmp/.X11-unix/X(display)*/
char *nameofit;			/*where (display) is a number*/
{
if (unix_sock!=0)
	{
	nameofit=strcat(X_UNIX_PATH,nameofit);
	unlink(nameofit);
	}
}

int econ_handler()		/*This gets rid of a fd set on which one */
{				/*either was hungup, closed, or reset,etc */
int i,j,k;
aio_cancel(fds[fdset].aio[0]->aio_fildes,fds[fdset].aio[0]);	/*cancel any outstanding asynch read calls*/
aio_cancel(fds[fdset].aio[1]->aio_fildes,fds[fdset].aio[1]);	/*on these two */
free((void *)fds[fdset].aio[0]->aio_buf);	/*free the mallocted space*/
free((void *)fds[fdset].aio[1]->aio_buf);
close(fds[fdset].aio[0]->aio_fildes);	/*close up the connections*/
close(fds[fdset].aio[1]->aio_fildes);
free(fds[fdset].aio[0]);
free(fds[fdset].aio[1]);
for (i=fdset;i<fdsets;i++)	/*shuffle the array of fd sets */
	fds[i]=fds[i+1];
fdsets--;
return -1;
}

void check_descriptors()	/*SIGIO handler - at least one asynch read*/
{				/* has completed */
int i,temp1,temp2,block,found;
int return0,return1;
struct timeval timet;

block=sigblock(signalmask);
timerclear(&timet);		/* zero the timeval struct */
for (i=0,errno=1;
	((fdsets>=0)&&(i<=fdsets));
	i++,errno=1)	/*cycle through fd sets searching for return*/
	{
	if ((((return0=aio_error(fds[i].aio[0]))!=EINPROGRESS)||
		((return1=aio_error(fds[i].aio[1]))!=EINPROGRESS))&&
		((return0!=ECANCELED)&&(return1!=ECANCELED)))
		{
		fdset=i;	/*set global fd set variable to i*/
		if (((return0!=EINPROGRESS)&&(return0!=0))||
			((return1!=EINPROGRESS)&&(return1!=0)))
			econ_handler();
		/*ditch the current fd set - */
		/*socket was closed or hung-up*/
		else	{
			if (return0==0)
				{
				if (fds[i].aiotable[0] == 0) {
					fds[i].aiotable[0]=1;
			fds[i].offset[0]=fds[i].aio[0]->aio_nbytes=aio_return(fds[i].aio[0]);
					fds[i].aio[0]->aio_fildes=fds[i].fd[1];
					fds[i].aio[0]->aio_offset=0;
					fds[i].aio[0]->aio_reqprio=AIO_PRIO_DFL;
					fds[i].aio[0]->aio_sigevent.sigev_signo=SIGIO;
					aio_write(fds[i].aio[0]);
					 }
				else {
				if ((return0=aio_return(fds[i].aio[0]))+
				fds[i].written[0]==fds[i].offset[0]) {
					fds[i].aiotable[0]=0;
				fds[i].written[0]=fds[i].offset[0]=0;
					fds[i].aio[0]->aio_fildes=fds[i].fd[0];
					fds[i].aio[0]->aio_offset=0;
					fds[i].aio[0]->aio_nbytes=12288;
					fds[i].aio[0]->aio_reqprio=AIO_PRIO_DFL;
					fds[i].aio[0]->aio_sigevent.sigev_signo=SIGIO;
					aio_read(fds[i].aio[0]);
					}
					else {
					fds[i].aio[0]->aio_offset=0;
					fds[i].aiotable[0]=1;
					fds[i].written[0]+=return0;
					fds[i].aio[0]->aio_nbytes-=return0;
					fds[i].aio[0]->aio_reqprio=AIO_PRIO_DFL;
					fds[i].aio[0]->aio_sigevent.sigev_signo=SIGIO;
					fds[i].aio[0]->aio_fildes=fds[i].fd[1];
					aio_write(fds[i].aio[0]);
						 }
					 }
				}
			else if (return1==0)
				{
				if (fds[i].aiotable[1] == 0) {
					fds[i].aiotable[1]=1;
			fds[i].offset[1]=fds[i].aio[1]->aio_nbytes=aio_return(fds[i].aio[1]);
					fds[i].aio[1]->aio_fildes=fds[i].fd[0];
					fds[i].aio[1]->aio_offset=0;
					fds[i].aio[1]->aio_reqprio=AIO_PRIO_DFL;
					fds[i].aio[1]->aio_sigevent.sigev_signo=SIGIO;
					aio_write(fds[i].aio[1]);
					 }
				else {
				if ((return1=aio_return(fds[i].aio[1]))+
				fds[i].written[1]==fds[i].offset[1]) {
					fds[i].aiotable[1]=0;
				fds[i].written[1]=fds[i].offset[1]=0;
					fds[i].aio[1]->aio_fildes=fds[i].fd[1];
					fds[i].aio[1]->aio_offset=0;
					fds[i].aio[1]->aio_nbytes=12288;
					fds[i].aio[1]->aio_reqprio=AIO_PRIO_DFL;
					fds[i].aio[1]->aio_sigevent.sigev_signo=SIGIO;
					aio_read(fds[i].aio[1]);
					}
					else {
					fds[i].aio[1]->aio_offset=0;
					fds[i].aiotable[1]=1;
					fds[i].written[1]+=return1;
					fds[i].aio[1]->aio_nbytes-=return1;
					fds[i].aio[1]->aio_reqprio=AIO_PRIO_DFL;
					fds[i].aio[1]->aio_sigevent.sigev_signo=SIGIO;
					fds[i].aio[1]->aio_fildes=fds[i].fd[0];
					aio_write(fds[i].aio[1]);
						 }
					 }
				}
			}
		}
	}
sigsetmask(block);		/*enable interrupts */
}


static int nick_intr_bump_flag(sig)
int sig;
{
int i;
char *tmp2;
signal (sig,SIG_IGN); /*turn off this signal while processing */
if ((sig!=SIGCHLD)||(waitpid(daemon,NULL,WNOHANG)!=0))
	{
	signal(SIGIO,exit);
	for (i=0;i<=fdsets;i++)
		{
		shutdown(fds[i].fd[0],2); /*close ALL fd sets*/
		close(fds[i].fd[0]);
		shutdown(fds[i].fd[1],2);
		close(fds[i].fd[1]);
		free((void *)fds[i].aio[0]->aio_buf);	/*free individual space*/
		free((void *)fds[i].aio[1]->aio_buf);
		free(fds[i].aio[0]);
		free(fds[i].aio[1]);
		}
	tmp2=(char *)strtok(name,":"); 
	XChuckWinScreen(strtok(tmp2,".")); /*ditch unix socket*/
	free(fds);
	killpg(0,SIGKILL);
	}
signal(sig,(void *)nick_intr_bump_flag);
}

/* The following is the main routine that takes a pseudo-screen structure*/
/* and a listening socket, and makes the connections on that socket think */
/* the pseudo screen is a root window */

void XWintoDisplay(ps,display_name,new_name,fd,argc,argv) 
PseudoScreen *ps;
char *display_name;		/* most of this I copied*/
char *new_name;
long int fd;
int argc;
char **argv;
{
Display *dpy;
Screen *sp;
Window test;
Depth info,*info1;
XWindowAttributes attrib;
Visual temp;
static int lock;
long int cur;
	char *display,*file;
	long int i,j, k;			/* random iterator indexes */
	xConnClientPrefix client;	/* client information */
	xConnSetupPrefix prefix;
	union {
		xConnSetup *setup;
		char *failure;
		char *vendor;
		xPixmapFormat *sf;
		xWindowRoot *rp;
		xDepth *dp;
		xVisualType *vp;
	} u;				/* proto data returned from server */
	long int vendorlen;			/* length of vendor string */
	char *setup;			/* memory allocated at startup */
	char *fullname = NULL;		/* expanded name of display */
	long int idisplay;			/* display number */
	long int iscreen;			/* screen number */
	long setuplength;		/* number of bytes in setup message */
	Xauth *authptr = NULL;
	char *server_addr = NULL;
	long int server_addrlen = 0;
	char *conn_auth_name, *conn_auth_data;
	long int conn_auth_namelen, conn_auth_datalen;
	long int conn_family;
	unsigned long mask;
	extern Bool _XSendClientPrefix();
	extern long int _XConnectDisplay();
	extern XID _XAllocID();
	long int newproc,newfd,testint,testfd;
        struct sockaddr *addr;
	char *tmp;
	xWindowRoot *sp2;
	long int *addrlen;
	long int *temp3;
	Window *tmp1;
        FILE *fd2,*fd3;
        struct stat statbuf;
        char *soc1,*soc2,*tmp2,command_line[10];
	unsigned long nfds;
	long int allowed_open,block,arg=0,filebits,one=1;
	struct rlimit rlp;

if (setsid()==-1) exit(-1);	/*make this its own process group*/
if (getrlimit(RLIMIT_CPU,&rlp)!=-1)
	{
	rlp.rlim_cur=rlp.rlim_max;
	if (setrlimit(RLIMIT_CPU,&rlp)==-1)
		{
		fprintf(stderr,"\nCouldn't unlimit the CPUtime - This will\n");
		fprintf(stderr,"stop working when cputime is exhausted.\n");
		}
	}
if((daemon=fork())==-1) exit(-1); /*fork - daemon handles the sockets, */
if (daemon!=0) 			/* child handles window resize requests*/
	{
	name=new_name;		/*assign display name to global var*/
	for(i=0;(i<=31);i++)
		if ((i!=SIGCHLD)&&(i!=SIGHUP))
			signalmask|=sigmask(i);
	if (nick_exec_number!=0)/*if -f given, fork proc to handle*/
		{
		if ((exec_flag=fork())==-1)  exit(-1);
		else if (exec_flag==0)
			{
			signal(SIGTERM,exit);
			file = argv[nick_exec_number+1];
			strcpy(command_line,"-display\0");
			argv[0]=argv[nick_exec_number+1];
			argv[1]=command_line;
			argv[2]=new_name;
			for(i=nick_exec_number+2;i<argc;i++)
				argv[i-nick_exec_number+1]=argv[i];
			argc-=(nick_exec_number-1); 
			argv[argc]=NULL;
			if ((execvp(file,argv))==-1) exit(-1);
			}
		}
	(void) signal(SIGHUP,(void *) nick_intr_bump_flag);/*set signals*/
	(void) signal(SIGINT,(void *)nick_intr_bump_flag);
	(void) signal(SIGTERM,(void *)nick_intr_bump_flag);
	(void) signal(SIGPIPE,SIG_IGN);
	(void) signal(SIGCHLD,(void *)nick_intr_bump_flag);
	(void) signal(SIGIO,(void *)check_descriptors);
	fdsets=-1; 			/*initialize vars*/
	first_open=0;
	allowed_open=getdtablesize();	/*find max # file descriptors*/
	fds = (fdstruct *)malloc(((int)(allowed_open/2+1)) *sizeof(fdstruct));
	while (1)		/*start "accept" daemon*/
		{
		if ((first_open +2*fdsets + 3 <= allowed_open)&&
		 ((newfd=accept(fd, (struct sockaddr *) NULL, 
			(int *) NULL))>=0)) /*blocking accept*/
			{
			if (first_open==0) first_open=newfd;
			fdsets++;	/*increase # open fd sets*/
			LockMutex(&lock);	/*start communication*/
			if (_XErrorFunction == NULL)
				 (void) XSetErrorHandler (NULL);
			if (_XIOErrorFunction == NULL)
				 (void) XSetIOErrorHandler (NULL);
			if ((dpy = (Display *)Xcalloc(1,
				 sizeof(Display))) == NULL) 
				UnlockMutex(&lock);
			 for (i=0;(i<10)&&
				((dpy->fd = _XConnectDisplay (display_name,
				 &fullname, &idisplay,
				 &iscreen, &conn_family,
				 &server_addrlen, &server_addr)) < 0);
				i++);
			if (dpy->fd<0)
				{
				Xfree ((char *) dpy);
				UnlockMutex(&lock);
				killpg(0,SIGTERM);
			}
			if (xauth_name && xauth_data) {
			    conn_auth_namelen = xauth_namelen;
			    conn_auth_name = xauth_name;
			    conn_auth_datalen = xauth_datalen;
			    conn_auth_data = xauth_data;
			} else {
			    char dpynumbuf[40];	
			/* big enough to hold 2^64 and more */
			    (void) sprintf (dpynumbuf, "%d", idisplay);		
				conn_auth_namelen = 0;
				conn_auth_name = NULL;
				conn_auth_datalen = 0;
				conn_auth_data = NULL;
			}
			if (server_addr) (void) Xfree (server_addr);
			readme(newfd,(char *) &client,
				 (long) SIZEOF(xConnClientPrefix));
			if (!_XSendClientPrefix(dpy, &client,
				 conn_auth_name, conn_auth_data))
			{
			    _XDisconnectDisplay (dpy->fd);
			    Xfree ((char *)dpy);
			    UnlockMutex(&lock);
			    killpg(0,SIGTERM);
			}	    
			if (authptr) XauDisposeAuth (authptr);
			_XRead (dpy, (char *)&prefix,
				(long)SIZEOF(xConnSetupPrefix));
			if(client.byteOrder==108)
				setuplength=change_endian(prefix.length,2)<<2;
			else 
				setuplength = prefix.length << 2;
			if ( (u.setup = (xConnSetup *)
			(setup =  Xmalloc ((unsigned long) setuplength))) == NULL)
			{
				_XDisconnectDisplay (dpy->fd);
				Xfree ((char *)dpy);
				UnlockMutex(&lock);
				killpg(0,SIGTERM);
			}
			_XRead (dpy, (char *)u.setup, setuplength);
			if(u.setup->numRoots > 1) 
				fprintf(stderr,"\nSet only for screen 0\n");
			prefix.success=1;
			if(client.byteOrder==108)
				{
				tmp=(char *)u.setup;
				tmp+=(32+change_endian(u.setup->nbytesVendor,2)+
				pad(change_endian(u.setup->nbytesVendor,2))+
				8*u.setup->numFormats);
				temp3=(long int *)&ps->root;
				*((long int *)tmp)=change_endian(*temp3,4);
				}
			else if(client.byteOrder==66)
				{
				tmp=(char *)u.setup; /*these lines replace the*/
				tmp+=(32+u.setup->nbytesVendor+
				pad(u.setup->nbytesVendor)+
				8*u.setup->numFormats);
				temp3=(long int *)&ps->root; /*root window id*/
				*((long int *)tmp)=*temp3;
				/*which is why this whole thing works!*/
				}
/*block ALL signals for a minute*/	block=sigblock(signalmask);
			writeme(newfd,(char *) &prefix, 
				(long) SIZEOF(xConnSetupPrefix));
			writeme(newfd,(char *) u.setup,
				(long) setuplength);
			ioctl(newfd,FIONBIO,&arg); 
			ioctl(dpy->fd,FIONBIO,&arg);
			fds[fdsets].fd[0]=newfd;
			fds[fdsets].fd[1]=dpy->fd;
			fds[fdsets].aio[0]= (struct aiocb *)malloc(sizeof(struct aiocb));
			fds[fdsets].aio[1]= (struct aiocb *)malloc(sizeof(struct aiocb));
			fds[fdsets].aio[0]->aio_buf=(char *)malloc(12288);
			fds[fdsets].aio[1]->aio_buf=(char *)malloc(12288);
			fds[fdsets].aio[0]->aio_fildes=newfd;
			fds[fdsets].aio[1]->aio_fildes=dpy->fd;
			fds[fdsets].aio[0]->aio_offset=0;
			fds[fdsets].aio[1]->aio_offset=1;
			fds[fdsets].aio[0]->aio_nbytes=12288;
			fds[fdsets].aio[1]->aio_nbytes=12288;
			fds[fdsets].aio[0]->aio_reqprio=AIO_PRIO_DFL;
			fds[fdsets].aio[1]->aio_reqprio=AIO_PRIO_DFL;
			fds[fdsets].aio[0]->aio_sigevent.sigev_signo=SIGIO;
			fds[fdsets].aio[1]->aio_sigevent.sigev_signo=SIGIO;
			fds[fdsets].aiotable[0]=
			fds[fdsets].aiotable[1]=
			fds[fdsets].offset[0]=
			fds[fdsets].offset[1]=
			fds[fdsets].written[0]=
			fds[fdsets].written[1]=0;
/*start asynch reads!*/	aio_read(fds[fdsets].aio[0]);
			aio_read(fds[fdsets].aio[1]);
			sigsetmask(block);
			}
		}
	}	/*daemon returning from parent process  - if daemon !=0 */
}	/* XWintoScreen() */

void readme(fd, buf, nbyte) /*little atomic read function*/
long int fd,nbyte;
char *buf;
{
long int i=nbyte,j,k;
j=k=0;
while (i)
	{
	if ((j=read(fd,buf+k,i))!=-1)
		{
		i-=j;
		k+=j;
		}
	}
}

void writeme(fd, buf, nbyte) /*little atomic write function*/
long int fd,nbyte;
char *buf;
{
long int i=nbyte,j,k;
j=k=0;
while (i)
	{
	if ((j=write(fd,buf+k,i))!=-1)
		{
		i-=j;
		k+=j;
		}
	}
}
